
/* ------------------------------- fileio.c -------------------------------- */

#include "main.h"
#include "fileio.h"

/* functions to read/write soundfiles */

void	ReadBlock (int pos, float *xblock)
{
	int	i, j, k;

	if (pos < 0) {		/* initialize buffer */

		for (i = 0; i < (MAXBLKSIZE << 3); i++) xblock_save[i] = 0.0;

		j = (MAXBLKSIZE << 2) * sizeof (float);
		k = (MAXBLKSIZE << 2);

		while (j > 0) {

			i = read (infile, xblock_save + k,
					(j > 4096 ? 4096 : j) );
			if (i == -1) {
				fprintf (stderr, "%s: error reading %s: %s\n",
						argv[0], argv[1],
						strerror (errno) );
				exit (-1);
			}

			fsize2 += ((i / sizeof (float)) >> 1);
			if (i == 0) fsize = fsize2;
			j -= 4096;
			k += (i / sizeof (float));
		}
	}

	/* copy saved data to beginning of buffer */

	for (i = 0; i < (ALLBLKSIZE << 1); i++) xblock[i] = 0.0;
	for (i = 0; i < (MAXBLKSIZE << 3); i++) xblock[i] = xblock_save[i];

	/* read data from file */

	j = (ALLBLKSIZ2 << 1) * sizeof (float);
	k = (MAXBLKSIZE << 3);

	while (j > 0) {

		i = read (infile, xblock + k, (j > 4096 ? 4096 : j) );
		if (i == -1) {
			fprintf (stderr, "%s: error reading %s: %s\n",
					argv[0], argv[1],
					strerror (errno) );
			exit (-1);
		}

		fsize2 += ((i / sizeof (float)) >> 1);
		if (i == 0) fsize = fsize2;
		j -= 4096;
		k += (i / sizeof (float));
	}

	/* save end of buffer */

	for (i = 0; i < (MAXBLKSIZE << 3); i++)
		xblock_save[i] = xblock[i + (ALLBLKSIZ2 << 1)];
}

void	WriteBlock (int pos, float *xblock)
{
	int	i, j, k;
	float	x, y, d;

	/* pos =  file position in samples */

	d = clpl2 - clpl1;

	/* clip and scale output */

	for (i = 0; i < (ALLBLKSIZ2 << 1); i++) {

		x = (xblock[i] < 0.0 ? -1.0 : 1.0);	/* x = sign */
		y = fabs (xblock[i]);			/* y = abs. value */

		y = (y > clpl1 ?
			(y > (clpl1 + 0.5 * PI * d) ?
				clpl2
				: (clpl1 + d * sin ((y - clpl1) / d) ) )
			: y );

		xblock[i] = scale * x * y;
	}

	/* j = number of bytes to write to output file */

	j = (ALLBLKSIZ2 << 1) * sizeof (float);
	k = 0;

	if (pos > (fsize - ALLBLKSIZ2))
		j = ((fsize - pos) << 1) * sizeof (float);

	while (j > 0) {
		i = write (outfile, xblock + k, (j > 4096 ? 4096 : j));
		if (i == -1) {
			fprintf (stderr, "%s: error writing to %s: %s\n",
					argv[0], argv[2], strerror (errno) );
			exit (-1);
		}
		j -= 4096;
		k += (4096 / sizeof (float));
	}
}


