
/* ---------------------------------------- getparams.c ---------------------------------------- */

#include "main.h"
#include "getparams.h"

/* --------------------------------------------------------------------------------------------- */

void	PrintVersion (void)		/* print program version */
{
	fprintf (stderr, "maximizer/limiter v1.1.0 (Jul 10 2000)\n\n");
}

void	PrintUsage (void)		/* print program usage */
{
	fprintf (stderr, "Usage: %s <infile> <outfile> [options...]\n", argv[0]);
	fprintf (stderr, "       infile must be raw, stereo, PCM float data\n");
	fprintf (stderr, "       use \"-\" to read from stdin or write to stdout\n\n");

	fprintf (stderr, "options:\n");

	fprintf (stderr, "       -verbose\t\t\t\t\t(default: off)\n\n");

	fprintf (stderr, "       -lmode <0/1: limiter mode off/on>\t(default: %2d)\n",	l_mode);
	fprintf (stderr, "       (note: options marked with \"*\" are ignored in limiter mode)\n");
	fprintf (stderr, "\n");

	fprintf (stderr, "       -srate <sample rate>\t\t\t(default: %9.2f)\n",		esr);
	fprintf (stderr, "       -limit <max. amp.>\t\t\t(default: %9.2f)\n",		limit);
	fprintf (stderr, "       -clpl1 <clip level 1>\t\t\t(default: %9.2f)\n",	clpl1);
	fprintf (stderr, "       -clpl2 <clip level 2>\t\t\t(default: %9.2f)\n",	clpl2);
	fprintf (stderr, "       -scale <output scale>\t\t\t(default: %9.2f)\n\n",	scale);

	fprintf (stderr, " *     -freq1 <filter 1 frequency>\t\t(default: %9.2f)\n",	ffreq1);
	fprintf (stderr, " *     -flvl1 <filter 1 level / low>\t\t(default: %9.2f)\n",	flvl1);
	fprintf (stderr, " *     -flvh1 <filter 1 level / high>\t\t(default: %9.2f)\n",	flvh1);
	fprintf (stderr, "\n");

	fprintf (stderr, " *     -freq2 <filter 2 frequency>\t\t(default: %9.2f)\n",	ffreq2);
	fprintf (stderr, " *     -flvl2 <filter 2 level / low>\t\t(default: %9.2f)\n",	flvl2);
	fprintf (stderr, " *     -flvh2 <filter 2 level / high>\t\t(default: %9.2f)\n",	flvh2);
	fprintf (stderr, "\n");

	fprintf (stderr, " *     -freq3 <filter 3 frequency>\t\t(default: %9.2f)\n",	ffreq3);
	fprintf (stderr, " *     -flvl3 <filter 3 level / low>\t\t(default: %9.2f)\n",	flvl3);
	fprintf (stderr, " *     -flvh3 <filter 3 level / high>\t\t(default: %9.2f)\n",	flvh3);
	fprintf (stderr, "\n");

	fprintf (stderr, " *     -freq4 <filter 4 frequency>\t\t(default: %9.2f)\n",	ffreq4);
	fprintf (stderr, " *     -flvl4 <filter 4 level / low>\t\t(default: %9.2f)\n",	flvl4);
	fprintf (stderr, " *     -flvh4 <filter 4 level / high>\t\t(default: %9.2f)\n",	flvh4);
	fprintf (stderr, "\n");

	fprintf (stderr, "       -lband <min. band number>\t\t(default: %2d)\n",	lband);
	fprintf (stderr, "       -nband <number of bands>\t\t\t(default: %2d)\n",	numbands);
	fprintf (stderr, "       (note: band numbers should be in the range -2..9)\n\n");

	fprintf (stderr, "       -smode <0: independent channels, 1: joint stereo>\n");
	fprintf (stderr, "       -s_mode <band number> <0/1>\n");
	fprintf (stderr, "        (default:\t"); PrintArrayInt (stereo_mode);
	fprintf (stderr, ")\n\n");

	fprintf (stderr, "       -ovrlp <overlap number for all bands>\n");
	fprintf (stderr, "       -overlap <band number> <overlap>\n");
	fprintf (stderr, "        (default:\t"); PrintArrayInt (ovrlap);
	fprintf (stderr, ")\n\n");

	fprintf (stderr, " *     -clpsn <band number (a: all)> <clip sensitivity>\n");
	fprintf (stderr, "        (default:"); PrintArrayFloat (clp_sn, "\t", "\t\t\t");
	fprintf (stderr, ")\n");
	fprintf (stderr, "       -r_lim <band number (a: all)> <rel. limit>\n");
	fprintf (stderr, "        (default:"); PrintArrayFloat (rel_limit, "\t", "\t\t\t");
	fprintf (stderr, ")\n");
	fprintf (stderr, " *     -o_res <band number (a: all)> <offs. resolution>\n");
	fprintf (stderr, "        (default:"); PrintArrayFloat (offs_res, "\t", "\t\t\t");
	fprintf (stderr, ")\n");
	fprintf (stderr, " *     -o_max <band number (a: all)> <max. offset>\n");
	fprintf (stderr, "        (default:"); PrintArrayFloat (offs_max, "\t", "\t\t\t");
	fprintf (stderr, ")\n\n");
}

/* --------------------------------------------------------------------------------------------- */

/* print settings for all bands */

void	PrintArrayFloat (float *x, char *p1, char *p2)
{
	fprintf (stderr, "%s%7.4f, %7.4f, %7.4f, %7.4f, %7.4f, %7.4f,\n",
			p1, x[0], x[1], x[2], x[3], x[4], x[5]);
	fprintf (stderr, "%s%7.4f, %7.4f, %7.4f, %7.4f, %7.4f, %7.4f",
			p2, x[6], x[7], x[8], x[9], x[10], x[11]);
}

void	PrintArrayInt (int *x)
{
	fprintf (stderr, "%2d, %2d, %2d, %2d, %2d, %2d,", x[0], x[1], x[2], x[3], x[4], x[5]);
	fprintf (stderr, " %2d, %2d, %2d, %2d, %2d, %2d", x[6], x[7], x[8], x[9], x[10], x[11]);
}

void	ErrMsg (char *errmsg)	/* error message */
{
	fprintf (stderr, "%s: %s\n", argv[0], errmsg);
	exit (-1);
}

void	ErrMsg2 (void)		/* print usage and exit (bad parameters) */
{
	PrintUsage ();
	exit (-1);
}

/* check if x is in the range min - max */

void	CheckRangeFloat (char *opt, float x, float min, float max)
{
	if (x < min) {
		fprintf (stderr, "%s: the min. allowed value for %s is %f\n",
				argv[0], opt, min);
		exit (-1);
	}

	if (x > max) {
		fprintf (stderr, "%s: the max. allowed value for %s is %f\n",
				argv[0], opt, max);
		exit (-1);
	}
}

void	CheckRangeInt (char *opt, int x, int min, int max)
{
	if (x < min) {
		fprintf (stderr, "%s: the min. allowed value for %s is %d\n",
				argv[0], opt, min);
		exit (-1);
	}

	if (x > max) {
		fprintf (stderr, "%s: the max. allowed value for %s is %d\n",
				argv[0], opt, max);
		exit (-1);
	}
}

/* Read a float value from command line					*/
/*	arg: parameter number						*/
/*	opt: option name						*/
/*	var: output variable						*/
/*	min: minimum value						*/
/*	max: max. value							*/

void	GetFloat (int *arg, char *opt, float *var, float min, float max)
{
	if ((*arg < argc) && ( ! (strcmp (argv[*arg], opt)))) {

		(*arg)++; if (*arg >= argc) ErrMsg2 ();
		*var = (float) atof (argv[*arg]);
		CheckRangeFloat (opt, *var, min, max);
		(*arg)++;
	}
}

/* Read a float value for one or all bands				*/

void	GetFloatForBand (int *arg, char *opt, float *var, float min, float max)
{
	int	i, i1, i2;
	float	x;

	if ((*arg < argc) && ( ! (strcmp (argv[*arg], opt)))) {

		(*arg)++; if (*arg >= argc) ErrMsg2 ();

		if ( ! (strcmp (argv[*arg], "a"))) {
			i1 = -2 ; i2 = 9;
		} else {
			i1 = i2 = (int) atoi (argv[*arg]);
		}

		if ((i1 < -2) || (i1 > 9)) {

			fprintf (stderr, "%s: band numbers should be in the range -2..9\n",
					argv[0]);
			exit (-1);
		}

		(*arg)++; if (*arg >= argc) ErrMsg2 ();
		x = (float) atof (argv[*arg]);
		CheckRangeFloat (opt, x, min, max);

		for (i = i1; i <= i2; i++) var[i+2] = x;
		(*arg)++;
	}
}

/* Read a float value for all bands					*/

void	GetFloatForAllBands (int *arg, char *opt, float *var, float min, float max)
{
	int	i, i1, i2;
	float	x;

	if ((*arg < argc) && ( ! (strcmp (argv[*arg], opt)))) {

		i1 = -2; i2 = 9;

		(*arg)++; if (*arg >= argc) ErrMsg2 ();
		x = (float) atof (argv[*arg]);
		CheckRangeFloat (opt, x, min, max);

		for (i = i1; i <= i2; i++) var[i+2] = x;
		(*arg)++;
	}
}

/* Read an integer value from command line				*/
/*	arg: parameter number						*/
/*	opt: option name						*/
/*	var: output variable						*/
/*	min: minimum value						*/
/*	max: max. value							*/

void	GetInt (int *arg, char *opt, int *var, int min, int max)
{
	if ((*arg < argc) && ( ! (strcmp (argv[*arg], opt)))) {

		(*arg)++; if (*arg >= argc) ErrMsg2 ();
		*var = (int) atoi (argv[*arg]);
		CheckRangeInt (opt, *var, min, max);
		(*arg)++;
	}
}

/* Read an integer value for one or all bands				*/

void	GetIntForBand (int *arg, char *opt, int *var, int min, int max)
{
	int	i, i1, i2;
	int	x;

	if ((*arg < argc) && ( ! (strcmp (argv[*arg], opt)))) {

		(*arg)++; if (*arg >= argc) ErrMsg2 ();

		if ( ! (strcmp (argv[*arg], "a"))) {
			i1 = -2 ; i2 = 9;
		} else {
			i1 = i2 = (int) atoi (argv[*arg]);
		}

		if ((i1 < -2) || (i1 > 9)) {

			fprintf (stderr, "%s: band numbers should be in the range -2..9\n",
					argv[0]);
			exit (-1);
		}

		(*arg)++; if (*arg >= argc) ErrMsg2 ();
		x = (int) atoi (argv[*arg]);
		CheckRangeInt (opt, x, min, max);

		for (i = i1; i <= i2; i++) var[i+2] = x;
		(*arg)++;
	}
}

/* Read an integer value for all bands					*/

void	GetIntForAllBands (int *arg, char *opt, int *var, int min, int max)
{
	int	i, i1, i2;
	int	x;

	if ((*arg < argc) && ( ! (strcmp (argv[*arg], opt)))) {

		i1 = -2; i2 = 9;

		(*arg)++; if (*arg >= argc) ErrMsg2 ();
		x = (int) atoi (argv[*arg]);
		CheckRangeInt (opt, x, min, max);

		for (i = i1; i <= i2; i++) var[i+2] = x;
		(*arg)++;
	}
}

/* --------------------------------------------------------------------------------------------- */

void	GetParams (void)		/* read parameters from command line */
{
	int	i, j;

	j = 3;				/* j = parameter number */

	do {	i = j;

		GetFloat (&j, "-srate", &esr, 0.000001, 1000000000.0);		/* sample rate   */
		GetFloat (&j, "-limit", &limit, 0.000001, 1000000000.0);	/* limit         */
		GetFloat (&j, "-clpl1", &clpl1, 0.000001, 1000000000.0);	/* clip level 1  */
		GetFloat (&j, "-clpl2", &clpl2, 0.000001, 1000000000.0);	/* clip level 2  */
		GetFloat (&j, "-scale", &scale, -1000000.0, 1000000.0);		/* output scale  */

		GetFloat (&j, "-freq1", &ffreq1, 1.0, 1000000.0);	/* filter 1 frequency	 */
		GetFloat (&j, "-flvl1", &flvl1, -1000.0, 1000.0);	/* filter 1 level/low	 */
		GetFloat (&j, "-flvh1", &flvh1, -1000.0, 1000.0);	/* filter 1 level/hi	 */

		GetFloat (&j, "-freq2", &ffreq2, 1.0, 1000000.0);	/* filter 2 frequency	 */
		GetFloat (&j, "-flvl2", &flvl2, -1000.0, 1000.0);	/* filter 2 level/low	 */
		GetFloat (&j, "-flvh2", &flvh2, -1000.0, 1000.0);	/* filter 2 level/hi	 */

		GetFloat (&j, "-freq3", &ffreq3, 1.0, 1000000.0);	/* filter 3 frequency	 */
		GetFloat (&j, "-flvl3", &flvl3, -1000.0, 1000.0);	/* filter 3 level/low	 */
		GetFloat (&j, "-flvh3", &flvh3, -1000.0, 1000.0);	/* filter 3 level/hi	 */

		GetFloat (&j, "-freq4", &ffreq4, 1.0, 1000000.0);	/* filter 4 frequency	 */
		GetFloat (&j, "-flvl4", &flvl4, -1000.0, 1000.0);	/* filter 4 level/low	 */
		GetFloat (&j, "-flvh4", &flvh4, -1000.0, 1000.0);	/* filter 4 level/hi	 */

		GetIntForAllBands (&j, "-ovrlp", ovrlap, 2, 256);	/* number of overlaps	 */
		GetIntForBand (&j, "-overlap", ovrlap, 2, 256);

		GetInt (&j, "-lband", &lband, -2, 9);			/* min. band number	 */
		GetInt (&j, "-nband", &numbands, 1, 12);		/* number of bands	 */

		GetFloatForBand (&j, "-clpsn", clp_sn, 0.01, 100.0);		/* clip sens.	 */
		GetFloatForBand (&j, "-r_lim", rel_limit, 0.001, 1000.0);	/* rel. limit	 */
		GetFloatForBand (&j, "-o_res", offs_res, 0.000001, 10.0);	/* offset res.	 */
		GetFloatForBand (&j, "-o_max", offs_max, 0.001, 1000.0);	/* max. offset	 */

		GetInt (&j, "-lmode", &l_mode, 0, 1);			/* limiter mode		 */

		GetIntForAllBands (&j, "-smode", stereo_mode, 0, 1);	/* j-stereo mode	 */
		GetIntForBand (&j, "-s_mode", stereo_mode, 0, 1);

		if ((j < argc) && ( ! (strcmp (argv[j], "-verbose")))) {

			v_mode = 1;					/* verbose mode		 */
			j++;
		}

		if ((j < argc) && (i == j)) ErrMsg2 ();			/* invalid parameter	 */

	} while (j < argc);

	lband += 2;
}

/* --------------------------------------------------------------------------------------------- */

void	CheckParameters (void)	/* check parameters and print them in case of verbose mode */
{
	int	i;

	if (clpl1 < limit) ErrMsg ("clip level 1 must be >= limit");
	if (clpl2 <= clpl1) ErrMsg ("clip level 2 must be > clip level 1");

	if (v_mode) {

		fprintf (stderr, "srate:\t\t%9.2f\n",		esr);
		fprintf (stderr, "limit:\t\t%9.2f\n",		limit);
		fprintf (stderr, "clip level 1:\t%9.2f\n",	clpl1);
		fprintf (stderr, "clip level 2:\t%9.2f\n",	clpl2);
		fprintf (stderr, "scale:\t\t%9.2f\n\n",		scale);

		fprintf (stderr, "rel_limit:"); PrintArrayFloat (rel_limit, "\t", "\t\t");
		fprintf (stderr, "\n\n");
	}

	if (!(l_mode)) {		/* the following parameters are not used in */
					/* limiter mode, so they will not be printed */

		if (	(ffreq1 >= (esr*0.5)) || (ffreq2 >= (esr*0.5))
		     || (ffreq3 >= (esr*0.5)) || (ffreq4 >= (esr*0.5)) )

		     	ErrMsg("filter frequency must be < srate/2");

		if (v_mode) {

			fprintf (stderr, "clip_sens:"); PrintArrayFloat (clp_sn, "\t", "\t\t");
			fprintf (stderr, "\n");

			fprintf (stderr, "offs_res:"); PrintArrayFloat (offs_res, "\t", "\t\t");
			fprintf (stderr, "\n");

			fprintf (stderr, "offs_max:"); PrintArrayFloat (offs_max, "\t", "\t\t");
			fprintf (stderr, "\n\n");

			fprintf (stderr, "filter 1 frequency:\t\t%9.2f\n",		ffreq1);
			fprintf (stderr, "filter 1 level / low freq:\t%9.2f\n",		flvl1);
			fprintf (stderr, "filter 1 level / high freq:\t%9.2f\n",	flvh1);

			fprintf (stderr, "filter 2 frequency:\t\t%9.2f\n",		ffreq2);
			fprintf (stderr, "filter 2 level / low freq:\t%9.2f\n",		flvl2);
			fprintf (stderr, "filter 2 level / high freq:\t%9.2f\n",	flvh2);

			fprintf (stderr, "filter 3 frequency:\t\t%9.2f\n",		ffreq3);
			fprintf (stderr, "filter 3 level / low freq:\t%9.2f\n",		flvl3);
			fprintf (stderr, "filter 3 level / high freq:\t%9.2f\n",	flvh3);

			fprintf (stderr, "filter 4 frequency:\t\t%9.2f\n",		ffreq4);
			fprintf (stderr, "filter 4 level / low freq:\t%9.2f\n",		flvl4);
			fprintf (stderr, "filter 4 level / high freq:\t%9.2f\n\n",	flvh4);
		}
	}

	if ((numbands + lband) > 12) ErrMsg ("max. band number should be less than 10");

	for (i = lband; i < (numbands + lband); i++) {

		if (	(ovrlap[i] != 2)
		     &&	(ovrlap[i] != 4)
		     &&	(ovrlap[i] != 8)
		     &&	(ovrlap[i] != 16)
		     &&	(ovrlap[i] != 32)
		     &&	(ovrlap[i] != 64)
		     &&	(ovrlap[i] != 128)
		     &&	(ovrlap[i] != 256) )

			ErrMsg ("number of overlaps must be power of two");

		if (ovrlap[i] > (1 << (12 - i)))

			ErrMsg ("number of overlaps must be <= 2 ^ (10 - band number)");
	}

	if (v_mode) {

		fprintf (stderr, "limiter mode:\t\t%2d\n", l_mode);
		fprintf (stderr, "min. band number:\t%2d\n", lband - 2);
		fprintf (stderr, "number of bands:\t%2d\n", numbands);
		fprintf (stderr, "number of overlaps:\t"); PrintArrayInt (ovrlap);
		fprintf (stderr, "\n");
		fprintf (stderr, "joint stereo mode:\t"); PrintArrayInt (stereo_mode);
		fprintf (stderr, "\n\n");

		fprintf (stderr, "input file: %s\n", argv[1]);
		fprintf (stderr, "output file: %s\n\n", argv[2]);
	}
}

/* --------------------------------------------------------------------------------------------- */


